#!/bin/bash

shellfu import pretty


#
# Beaker Task utils
#
# This module contains several functions to help with reporting
# task state and results from within a Beaker task.
#


#
# Root dir where to keep state
#
__BKRH__STATEDIR=${__BKRH__STATEDIR:-/var/tmp/bkrh}

#
# Content upload size
#
__BKRH__SUBMIT_REQSIZE=${__BKRH__SUBMIT_REQSIZE:-1024000}

bkrh__log() {
    #
    # Upload file $1 to task with name $2 or inferred
    #
    # Usage:
    #     bkrh__log PATH [NAME]
    #
    local Path=$1   # path to uploaded file
    local name=$2   # alternative name of uploaded file
    local ByteA=0   # address of first byte (0-indexed)
    local ByteZ     # address of last byte (0-indexed)
    local Size      # size of whole file in bytes
    local Url       # upload URL
    local Lim=$__BKRH__SUBMIT_REQSIZE
    Url="$(__bkrh__taskurl)logs/${name////-}"
    Size=$(stat -c %s "$Path")
    togo=$Size
    ByteA=0
    while test "$togo" -gt 0; do            # togo=23
        if test "$togo" -gt "$Lim"; then    # Lim=10                                                        Lim=100
            ByteZ=$((ByteA + Lim - 1))      # ByteZ:    o? | (0+10-1)  o9 | (10+10-1) o19
            __bkrh__logchunk
            togo=$((togo - Lim))            # togo:     s23 | (23-10) s13 | (13-10)   s3
            ByteA=$((ByteZ + 1))            # ByteA:     o0 | (9+1)   o10 | (19+1)    o20
        else
            ByteZ=$((ByteA + togo - 1))     # ByteZ:                                  o19 | (20+3-1) o22    (0+23-1) o22
            __bkrh__logchunk
            togo=0
        fi
    done
}

__bkrh__logchunk() {
    #
    # Upload part $1 to $2 of size $3 log file $4 to url $5
    #
    local cbuff     # current chunk buffer
    local es        # curl exit status
    local csize     # current chunk size
    cbuff=$(mktemp)
    <"$Path" tail -c "+$((ByteA+1))" \
      | head -c "$Lim" >"$cbuff"
    csize=$(stat -c %s "$cbuff")
    __bkrh__curl \
        -H "Content-Range: bytes $ByteA-$ByteZ/$Size" \
        -H "Content-Length: $csize" \
        -T "$cbuff" "$Url"; es=$?
    rm "$cbuff"
    return $es
}

bkrh__result() {
    #
    # Report result status
    #
    # Usage:
    #     bkrh__result [-m MESSAGE] [-p PATH] [-s SCORE] pass|fail|warning|error|none
    #
    local result
    local bkrresult
    local cmd=()
    local path
    local message
    local score
    while true; do case $1 in
        -m) message=$2;    shift 2 || return 2 ;;
        -p) path=$2;    shift 2 || return 2 ;;
        -s) score=$2;    shift 2 || return 2 ;;
        -*) warn "unknown argument: $1"; return 2 ;;
        *)  break ;;
    esac done
    result=$1
    case $result in
        pass)       bkrresult=Pass ;;
        fail)       bkrresult=Fail ;;
        warning)    bkrresult=Warn ;;
        error)      bkrresult=Warn ;;
        none)       bkrresult=None ;;
        "")         bkrresult=None ;;
        *)  warn "bad result (must be pass|fail|warning|error|none): $result"
    esac
    cmd=(
        __bkrh__taskresult \
            "result=$bkrresult" \
            "path=$path" \
            "score=$score" \
            "message=$message"
    )
    debug -v cmd
    "${cmd[@]}"
}

bkrh__status() {
    #
    # Set task status
    #
    # Usage:
    #     bkrh__status [-n NAME] [-v VERSION] running|completed|aborted
    #
    local status
    local name
    local version
    local cmd=()
    while true; do case $1 in
        -n) name=$2;    shift 2 || return 2 ;;
        -v) version=$2; shift 2 || return 2 ;;
        -*) warn "unknown argument: $1"; return 2 ;;
        *)  break ;;
    esac done
    status=$1
    cmd=(
        __bkrh__taskinfo \
            "status=${status^}"
    )
    test -n "$name" && cmd+=("name=$name")
    test -n "$version" && cmd+=("version=$version")
    debug -v cmd
    "${cmd[@]}"
}

bkrh__watchdog() {
    #
    # Extend recipe watchdog $1 from now
    #
    # Usage:
    #     bkrh__watchdog DURATION
    #
    # Watchdog kill time is extended to be now plus DURATION, where
    # DURATION must be provided in one of following forms:
    #
    #     NUMBERd
    #     NUMBERh
    #     NUMBERm
    #     NUMBERs
    #     NUMBER
    #
    # That is, an integer decimal NUMBER, followed by one of units:
    # 'd' for days, 'h' for hours, 'm' for minutes' and 's' for
    # seconds.  If unit is missing, seconds are assumed.
    #
    # Few examples:
    #
    #     2d    # two days
    #     14m   # 14 minutes
    #     99s   # 99 seconds
    #     99    # 99 seconds
    #
    # Units cannot be combined.
    #
    local duration=$1
    local seconds
    seconds=$(__bkrh__parse_duration "$duration") || {
        warn "invalid duration: '$duration'"
        return 3
    }
    __bkrh__curl \
        --data-urlencode "seconds=$seconds" \
        "$BEAKER_LAB_CONTROLLER_URL/recipes/$BEAKER_RECIPE_ID/watchdog"
}

__bkrh__taskinfo() {
    #
    # Encode and PATCH pairs $@ to task
    #
    local url
    local cargs=(-X PATCH)
    local pair
    for pair in "$@"; do
        cargs+=(--data-urlencode "$pair")
    done
    url=$(__bkrh__taskurl)
    cargs+=("$url")
    __bkrh__curl "${cargs[@]}"
}

__bkrh__parse_curl_log() {
    #
    # Parse and save useful data from latest curl log
    #
    # Location: http://lab-02.rhts.eng.brq.redhat.com:8000/recipes/7986573/tasks/107318632/results/494398189
    grep '^< Location:' "$Sd/curl.err" \
      | tail -1 \
      | grep -Eo 'http[[:graph:]]+' > "$Sd/location"
    <"$Sd/location" grep -Eo 'results/[0-9]+' \
      | cut -d/ -f2 > "$Sd/resultid.now"
    <"$Sd/location" grep -Eo 'tasks/[0-9]+' \
      | cut -d/ -f2 > "$Sd/taskid.now"
    <"$Sd/location" grep -Eo 'recipes/[0-9]+' \
      | cut -d/ -f2 > "$Sd/recipeid.now"
    test -s "$Sd/resultid.now" && mv "$Sd/resultid.now" "$Sd/resultid"
    test -s "$Sd/taskid.now"   && mv "$Sd/taskid.now"   "$Sd/taskid"
    test -s "$Sd/recipeid.now" && mv "$Sd/recipeid.now" "$Sd/recipeid"
    true
}

__bkrh__parse_duration() {
    #
    # Parse duration $1 and print seconds
    #
    local arg=$1
    local duration
    local d=0
    local h=0
    local m=0
    local s=0
    local re='[0-9]+[dhms]?'
    duration=$(grep -xE -e "$re" <<<"$arg") || {
        warn -l "error parsing duration: $duration" \
                " .. must match regex: '$re'"
        return 2
    }
    case $duration in
        *d) d=${duration%d} ;;
        *h) h=${duration%h} ;;
        *m) m=${duration%m} ;;
        *s) s=${duration%s} ;;
        *)  s=${duration} ;;
    esac
    echo $(( 86400*d + 3600*h + 60*m + s ))
}

__bkrh__taskresult() {
    #
    # Encode and POST pairs $@ to task
    #
    local url
    local cargs=(-X POST)
    for pair in "$@"; do
        cargs+=(--data-urlencode "$pair")
    done
    url="$(__bkrh__taskurl)results/"
    cargs+=("$url")
    debug -v cargs
    __bkrh__curl "${cargs[@]}"
}

__bkrh__curl() {
    #
    # A more useful curl
    #
    local cmd=(curl -vfsSL "$@")
    local Sd=$__BKRH__STATEDIR
    rm -rf "$Sd"; mkdir -p "$Sd"
    debug -v cmd
    "${cmd[@]}" >"$Sd/curl.out" 2>"$Sd/curl.err" || {
        warn -l "curl call failed:" \
             "  cmd: ${cmd[*]}" \
             "  fun: ${FUNCNAME[2]}"
        warn -f "$Sd/curl.out"
        warn -f "$Sd/curl.err"
        return 3
    }
    debug -f "$Sd/curl.out"
    debug -f "$Sd/curl.err"
    __bkrh__parse_curl_log
}

__bkrh__taskurl() {
    #
    # Print task URL
    #
    local taskid
    taskid=$(
        rurl="$BEAKER_LAB_CONTROLLER_URL/recipes/$BEAKER_RECIPE_ID/"
        # <task name="/distribution/upgrade/jats/reserve" role="None" version="0.0-1" id="105183430" result="New" status="Running" avg_time="43200" start_time="2020-01-22 22:39:32" duration="Time Remaining 10:46:47">
        curl -fsSL "$rurl" \
          | grep '<task ' \
          | grep 'status=.Running.' \
          | grep -o 'id=.[0-9]\+.' \
          | grep -o '[0-9]\+'
    )
    #FIXME: ^^ find a better way
    BEAKER_LAB_CONTROLLER_URL=${BEAKER_LAB_CONTROLLER_URL:-}
    BEAKER_RECIPE_ID=${BEAKER_RECIPE_ID:-_unknown_recipe_}
    taskid=${taskid:-_unknown_task_}
    echo -n "$BEAKER_LAB_CONTROLLER_URL"
    echo -n "/recipes/$BEAKER_RECIPE_ID"
    echo -n "/tasks/$taskid/"
}

#shellfu module-version=0.0.5
